#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using System.Reflection;
using System.Windows.Forms;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	
	public class DrawingUtils {
		
		private  static int getXByBarIdxDouble(double barDouble, ChartControl chart) {
			return (int)typeof(ChartControl).GetMethod("GetXByBarIdxDouble",BindingFlags.NonPublic|BindingFlags.Instance).Invoke(chart,new Object [] {barDouble});
	
		}
			
		public  static Point getEnd(ChartLine line, IndicatorBase  indicator) {
			ChartControl chart = indicator.ChartControl;
			if (line==null) return new Point(0,0);
			int x,y;
			if (chart.EquidistantBars==true) {
			
				double end_bar_double = (double)typeof(ChartLine).GetProperty("endBarDouble",BindingFlags.NonPublic | BindingFlags.Instance).GetValue(line,null);
				x = getXByBarIdxDouble(end_bar_double,chart);
				y = chart.GetYByValue(indicator,line.EndY);
				return new Point(x,y);
			}
			x = chart.GetXByTime(line.EndTime); 
			y = chart.GetYByValue(indicator,line.EndY);
			return new Point(x,y);
		}	
		public  static Point getStart (ChartLine line,IndicatorBase indicator) {
			ChartControl chart = indicator.ChartControl;
			if (line==null) return new Point(0,0);
			int x,y;
			if (chart.EquidistantBars==true) {
				double start_bar_double = (double)typeof(ChartObject).GetProperty("startBarDouble",BindingFlags.NonPublic | BindingFlags.Instance).GetValue(line,null);
				x = getXByBarIdxDouble(start_bar_double,chart);
				y = chart.GetYByValue(indicator,line.StartY);
				return new Point(x,y);
			}
			
			x = chart.GetXByTime(line.StartTime);
			y=  chart.GetYByValue(indicator,line.StartY);
			return new Point(x,y);
		}	
	}
	
	 public class Arc {
		public Point start;
		public double radius;
		public double angle1, angle2;
		public Arc(Point start, double radius, double angle1, double angle2) {
			this.start = start; this.radius = radius; 
			this.angle1 = angle1; this.angle2 = angle2;
		}

		public double[] getValue(double angle) {
			// x = rcos(theta), y = rsin(theta)
			return new double[] {start.X+radius*Math.Cos(angle*Math.PI/180),start.Y-radius*Math.Sin(angle*Math.PI/180)};	
		}
		
		public bool within(double angle) {
			if (angle2>=angle1) {
				return (angle<=angle2 && angle>=angle1);
			}
			else {
				return !(angle<=angle1 && angle>=angle2);
			}
		}
		
		
		private double[] intersectXY (Arc circle) {
			double r1 = this.radius;
			double x1  = start.X; double y1 = start.Y;
			double r2 = circle.radius;
			double x2 = circle.start.X; double y2 = circle.start.Y;
			
			if (x1==x2 && y1==y2) return null;
			
			double value = (r1*r1-r2*r2)-(x1*x1-x2*x2)-(y1*y1-y2*y2);
			double m,d,a,b,c,radical,x_1,x_2,y_1,y_2;
			if (y1!=y2) {
				 m = -(x1-x2)/(y1-y2);
				 d = -value/(2*(y1-y2));
				 a= (1+m*m);
				 b = 2*m*d-2*m*y1-2*x1;
				 c = -(r1*r1-y1*y1-x1*x1-d*d+2*y1*d);
				
				 radical = b*b-4*a*c;
				if (radical<0) return null; 
				 x_1 = (-b+Math.Sqrt(radical))/(2*a);
				 x_2 = (-b-Math.Sqrt(radical))/(2*a);
				 y_1 = m*x_1+d;
				 y_2 = m*x_2+d;
				if (radical==0) 
					return new double[] { x_1,y_1};
				return new double[] {x_1,y_1,x_2,y_2};
			}
			
			//if y1==y2, then x1!=x2
			 m = -(y1-y2)/(x1-x2);
			 d = -value/(2*(x1-x2));
			 a= (1+m*m);
			 b = (2*m*d-2*x1*m-2*y1);
			 c = (-r1*r1+d*d+y1*y1+x1*x1-2*x1*d);
			
			 radical = b*b-4*a*c;
			if (radical<0) return null; 
			 y_1 = (-b+Math.Sqrt(radical))/(2*a);
			 y_2 = (-b-Math.Sqrt(radical))/(2*a);
			 x_1 = m*y_1+d;
			 x_2 = m*y_2+d;
			if (radical==0) 
				return new double[] { x_1,y_1};
			return new double[] {x_1,y_1,x_2,y_2};
			
		}
		
		private double  AngleOfLine(double x1, double y1, double x2, double y2) {
			double m = -(y2-y1)/(x2-x1);
			double angle =  Math.Atan(m)*180/Math.PI;
			if (x2<x1) {
				return angle+180;
			}
			if (angle<0) {
				angle = 360+angle;
			}
			return angle;	
		}
		
		public static int Inside = 0;
		public static int Outside = 1;
		
		public double[] intersect (Arc arc,IndicatorBase ind) {
			double [] xy_intersection = intersectXY(arc);
			
			if (xy_intersection==null) return null;
			double a1 = AngleOfLine(start.X,start.Y,xy_intersection[0],xy_intersection[1]);
			if (xy_intersection.Length==2) {
				if (within(angle1))
					return new double[] {angle1};
				else
					return null;
			}
			
			
			double a2 = AngleOfLine(start.X,start.Y,xy_intersection[2],xy_intersection[3]);
			 
			
			if (within(a1) && within(a2)) 
				return new double[] {a1,a2};
			else if (within(a1))
				return new double[] {a1};
			else if (within(a2))
				return new double[] {a2};
			return null;	
		}

		
		public static Arc cut(Arc obj, Arc edge,int type,IndicatorBase ind)  {
			double [] angles = obj.intersect(edge,ind);
			double [] val1 = obj.getValue(obj.angle1);
			double[] val2  = obj.getValue(obj.angle2);
			if (angles==null && (distance(edge.start,val1)<edge.radius) && (distance(edge.start,val2)<edge.radius)) return obj;
			if (angles==null) return null;
			if (angles.Length==2) {
				double angle1 = Math.Min(angles[0],angles[1]);
				double angle2 = Math.Max(angles[0],angles[1]);
				double average  = averageAngle(angle1,angle2);
				double [] value = obj.getValue(average);
				double d = distance(edge.start,value);
				if ((d<edge.radius && type==Arc.Inside) || (d>edge.radius && type==Arc.Outside)) {
					return new Arc(obj.start,obj.radius,angle1,angle2);		
				}
				else {
					return new Arc(obj.start,obj.radius,angle2,angle1);	
				}
			}
			else {
				double angle1 = angles[0];
				double angle2 = obj.angle1;
				double angle3 = obj.angle2;
				
				double average1 = averageAngle(angle2,angle1);
				double average2 = averageAngle(angle1,angle3);
				double []value1 = obj.getValue(average1);
				double []value2 = obj.getValue(average2);
				double d1 = distance(edge.start,value1);
				double d2 = distance(edge.start,value2);

				if ((d1<edge.radius && type==Arc.Inside) || (d1>edge.radius && type==Arc.Outside)) {
					return new Arc(obj.start,obj.radius,angle2,angle1);
				}				
				if ((d2<edge.radius && type==Arc.Inside) || (d2>edge.radius && type==Arc.Outside)) {
					return new Arc(obj.start,obj.radius,angle1,angle3);
				}
			}
			return null;
		}	
		
		private static double distance(Point start, double[] pt) {
			return Math.Sqrt(Math.Pow(pt[0]-start.X,2) + Math.Pow(pt[1]-start.Y,2));
		}
		
		private static double averageAngle(double angle1,double angle2) {
			
			if (angle2>=angle1) 
				return (angle1+angle2)/2.0;
			else 
				return (angle1+angle2)/2.0-180.0;

			
		}
		
		
		public static void drawArc (Graphics g , Pen pen, Arc arc) {
			if (arc==null) return;
			float startAngle = -(float)arc.angle1;
			float sweepAngle  = 0;
			if (arc.angle1>arc.angle2) 
				sweepAngle = -(float)(360-arc.angle1+arc.angle2);
			else
				sweepAngle = -(float)(arc.angle2-arc.angle1);
			
			g.DrawArc(pen,(int)(arc.start.X-arc.radius),(int)(arc.start.Y-arc.radius),(int)(2*arc.radius),(int)(2*arc.radius),startAngle,sweepAngle);
			
		}	
	}
	
	
	
	public class Circle:Arc {
		public static int Start = 0;
		public static int End = 1;
		public Circle (Point start, double radius):base(start,radius,0,360) {}
		public static Circle newCircle(IndicatorBase indicator,ChartLine line, int mode) {
			Point start = DrawingUtils.getStart(line,indicator);
			Point end = DrawingUtils.getEnd(line,indicator);
			//the start of the line is the one with the highest y value
			if (end.Y>start.Y) {
				Point temp = new Point(start.X,start.Y);
				start = new Point(end.X,end.Y); 
				end = temp;
			}
			
			double m;
			if (start.X==end.X) 
				m = Double.NaN;
			m = -(double)((start.Y-end.Y))/(start.X-end.X);
			Point pt;
			if (m>0) {
				pt = start;
			} else if (m<0) {
				pt = end;
			} else if (m==0) {
				if (end.X>start.X) 
					pt = start;
				else
					pt = end;
			} else {
				if (end.Y>start.Y)
					pt = start;
				else 
					pt = end;
			}
				
			double radius = Math.Sqrt(Math.Pow(start.X-end.X,2)+Math.Pow(start.Y-end.Y,2));
			if (mode==Circle.Start) {
				return new Circle(pt,radius);	
			}
			else if (mode==Circle.End) {
				return new Circle((pt==start?end:start),radius);
			}
			else {
				throw (new Exception("Invalid mode"));
			}			
		}	
	}
	
	
    public class CircleContours : Indicator
    {
        #region Variables
        // Wizard generated variables
        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Overlay				= true;
			CalculateOnBarClose = false; 
			ChartOnly   = true; 
        }

        protected override void OnBarUpdate()
        {
          
        }
		
	
	
	
	
		public ChartLine[] getLines (Color color) {
			int length = DrawObjects.Count;
			
			int number_of_lines = 0;
			foreach (IDrawObject obj in DrawObjects) {
				if (obj.DrawType == DrawType.Line && ((ChartLine)obj).Pen.Color==color) 
					++number_of_lines;
			}
			if (number_of_lines==0) return null;
			ChartLine[] objects = new ChartLine[number_of_lines];
			int index=0;
			foreach (IDrawObject obj in DrawObjects) {
				if (obj.DrawType ==DrawType.Line && ((ChartLine)obj).Pen.Color==color) {
					objects[index] = (ChartLine)obj; ++index;
				}
			}
			return objects;			
		}
		
		
		public override void Plot (Graphics g, Rectangle bounds, double min, double max) {
			try {
				g.SmoothingMode = SmoothingMode.AntiAlias;
				ChartLine[] objects = getLines(Color.PaleGreen);
				Pen pen = new Pen(new SolidBrush(Color.PaleGreen));
				pen.Width = 2;
				if (objects.Length==3) {
					ChartLine line1 = objects[0];
					ChartLine line2 = objects[1];
					ChartLine line3 = objects[2];
					
					Circle c1 = Circle.newCircle(this,line1,Circle.Start);
					Circle c2 = Circle.newCircle(this,line2,Circle.Start);
					Circle c3 = Circle.newCircle(this,line3,Circle.Start);
					
					Circle c1_green = new Circle(c1.start,c1.radius*2.0/3.0);
					Circle c1_red = new Circle(c1.start,c1.radius*1.5/3.0);
					
					Circle c2_green = new Circle(c2.start,c2.radius*2.0/3.0);
					Circle c2_red = new Circle(c2.start,c2.radius*1.5/3.0);
					
					Circle c3_green = new Circle(c3.start,c3.radius*2.0/3.0);
					Circle c3_red = new Circle(c3.start,c3.radius*1.5/3.0);
					
					//draw arc3 intersection
					Arc arc1 = Arc.cut(c1,c2,Arc.Inside,this);					
					if (arc1!=null)
						arc1 = Arc.cut(arc1,c3,Arc.Inside,this);
					
					Arc arc2 = Arc.cut(c2,c1,Arc.Inside,this);
					if (arc2!=null) 
						arc2 = Arc.cut(arc2,c3,Arc.Inside,this);	
					
					Arc arc3 = Arc.cut(c3,c1,Arc.Inside,this);
					if (arc3!=null)
						arc3 = Arc.cut(arc3,c2,Arc.Inside,this);
					
					
					Arc.drawArc(g,pen,arc1);
					Arc.drawArc(g,pen,arc2);
					Arc.drawArc(g,pen,arc3);
					
					//draw second arc intersection
				    arc1 = Arc.cut(c1_green,c2_green,Arc.Inside,this);					
					if (arc1!=null)
						arc1 = Arc.cut(arc1,c3_green,Arc.Inside,this);
					
				    arc2 = Arc.cut(c2_green,c1_green,Arc.Inside,this);
					if (arc2!=null) 
						arc2 = Arc.cut(arc2,c3_green,Arc.Inside,this);	
					
				    arc3 = Arc.cut(c3_green,c1_green,Arc.Inside,this);
					if (arc3!=null)
						arc3 = Arc.cut(arc3,c2_green,Arc.Inside,this);
					
					pen.Color = Color.LimeGreen;
					Arc.drawArc(g,pen,arc1);
					Arc.drawArc(g,pen,arc2);
					Arc.drawArc(g,pen,arc3);
					
					//draw third arc intersection
				    arc1 = Arc.cut(c1_red,c2_red,Arc.Inside,this);					
					if (arc1!=null)
						arc1 = Arc.cut(arc1,c3_red,Arc.Inside,this);
					
				    arc2 = Arc.cut(c2_red,c1_red,Arc.Inside,this);
					if (arc2!=null) 
						arc2 = Arc.cut(arc2,c3_red,Arc.Inside,this);	
					
				    arc3 = Arc.cut(c3_red,c1_red,Arc.Inside,this);
					if (arc3!=null)
						arc3 = Arc.cut(arc3,c2_red,Arc.Inside,this);
					
					pen.Color = Color.Red;
					Arc.drawArc(g,pen,arc1);
					Arc.drawArc(g,pen,arc2);
					Arc.drawArc(g,pen,arc3);
					
				}
				 
			} catch (Exception e) {
				Print(e.ToString()); 
			}
			
		}
		

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Plot0
        {
            get { return Values[0]; }
        }

        #endregion
    }
}
